@@ -6,28 +6,28 @@ acquisition. Used with some ImageAnalysers to receive images and to react to sta
 acquisitions.
 """
 
-from ctypes import alignment
 import json
 import logging
 import re
 import time
+from typing import Union
 import numpy as np
 
 import zmq
 from pycromanager import Bridge
 from qtpy.QtCore import QObject, QThread, Signal, Slot
 
-from .data_structures import MMSettings
-from .data_structures import PyImage
+from pymm_eventserver.data_structures import MMSettings
+from pymm_eventserver.data_structures import PyImage
 
 log = logging.getLogger("EDA")
 SOCKET = "5556"
 
 
 class EventThread(QObject):
-    """Thread that receives events from Micro-Manager and relays them to the main program. """
+    """Thread that receives events from Micro-Manager and relays them to the main program."""
 
-    def __init__(self, live_images: bool = False):
+    def __init__(self, live_images: bool = False, topics: Union[str, list] = "all"):
         """Set up the bridge to Micro-Manager, ZMQ sockets and the main listener Thread."""
         super().__init__()
 
@@ -48,27 +48,32 @@ class EventThread(QObject):
         self.context = zmq.Context()
         self.socket = self.context.socket(zmq.SUB)
         self.socket.connect("tcp://localhost:" + SOCKET)
-        self.socket.setsockopt(zmq.RCVTIMEO, 10000)  # Timeout for the recv() function
+        self.socket.setsockopt(zmq.RCVTIMEO, 1000)  # Timeout for the recv() function
 
         self.thread_stop = False
 
-        self.topics = [
-            "StandardEvent",
-            "GUIRefreshEvent",
-            "LiveMode",
-            "Acquisition",
-            "GUI",
-            "Hardware",
-            "Settings",
-            "NewImage",
-        ]
+        if not isinstance(topics, list):
+            self.topics = [
+                "StandardEvent",
+                "GUIRefreshEvent",
+                "LiveMode",
+                "Acquisition",
+                "GUI",
+                "Hardware",
+                "Settings",
+                "NewImage",
+            ]
+        else:
+            self.topics = topics
+
         for topic in self.topics:
             self.socket.setsockopt_string(zmq.SUBSCRIBE, topic)
 
         # Set up the main listener Thread
         self.thread = QThread()
-        self.listener = EventListener(self.socket, self.event_sockets, self.bridge, self.thread,
-                                      self.live_images)
+        self.listener = EventListener(
+            self.socket, self.event_sockets, self.bridge, self.thread, self.live_images
+        )
         self.listener.moveToThread(self.thread)
         self.thread.started.connect(self.listener.start)
         self.listener.stop_thread_event.connect(self.stop)
@@ -104,9 +109,16 @@ class EventListener(QObject):
     live_mode_event = Signal(bool)
     xy_stage_position_changed_event = Signal(tuple)
     stage_position_changed_event = Signal(float)
+    exposure_changed_event = Signal(str, str, str)
 
-    def __init__(self, socket, event_sockets, bridge: Bridge, thread: QThread,
-                 send_live_images: bool = False):
+    def __init__(
+        self,
+        socket,
+        event_sockets,
+        bridge: Bridge,
+        thread: QThread,
+        send_live_images: bool = False,
+    ):
         """Store passed arguments and starting time for frequency limitation of certain events."""
         super().__init__()
         self.send_live_images = send_live_images
@@ -154,20 +166,13 @@ class EventListener(QObject):
                     image_bit = str(self.socket.recv())
                     # TODO: Maybe this should also be done for other bitdepths?!
                     image_depth = np.uint16 if image_bit == "b'2'" else np.uint8
+                    metadata = self.socket.recv()
+                    metadata_dict = json.loads(str(metadata)[11:-1])
                     next_message = self.socket.recv()
-                    print(next_message[:4])
-                    if str(next_message[:4]) == "b'Acqu'":  # AcquisitionEndedEvent interrupted
-                        print("Acquisition ENDED?")
-                        next_message = json.loads(re.split(" ", str(next_message))[1][0:-1])
-                        evt, eventString = self.translate_message(next_message, instance)
-                        print(eventString)
-                        image_message = self.socket.recv()
-                        print(image_message[:4])
-                        py_image = self.image_from_message(image_message, reply, image_depth)
-                        self.new_image_event.emit(py_image)
-                    else:
-                        py_image = self.image_from_message(next_message, reply, image_depth)
-                        self.new_image_event.emit(py_image)
+                    py_image = self.image_from_message(
+                        next_message, reply, image_depth, metadata_dict
+                    )
+                    self.new_image_event.emit(py_image)
 
                 print(eventString)
                 if "DefaultAcquisitionStartedEvent" in eventString:
@@ -179,14 +184,15 @@ class EventListener(QObject):
                 elif "DefaultAcquisitionEndedEvent" in eventString:
                     self.acquisition_ended_event.emit(evt)
                 elif "CustomSettingsEvent" in eventString:
-                    self.configuration_settings_event.emit(
-                        evt.get_device(), evt.get_property(), evt.get_value()
-                    )
+                    dev = evt.get_device()
+                    prop = evt.get_property()
+                    val = evt.get_value()
+                    self.configuration_settings_event.emit(dev, prop, val)
+                elif "DefaultExposureChangedEvent" in eventString:
+                    self.exposure_changed_event.emit(
+                        "exposure", "time_ms", str(evt.get_new_exposure_time()))
                 elif "DefaultStagePositionChangedEvent" in eventString:
-                    if (
-                        self.blockZ > 0
-                        or time.perf_counter() - self.last_stage_position < 0.05
-                    ):
+                    if self.blockZ > 0 or time.perf_counter() - self.last_stage_position < 0.05:
                         print("BLOCKED ", self.blockZ)
                     else:
                         self.stage_position_changed_event.emit(evt.get_pos() * 100)
@@ -196,7 +202,6 @@ class EventListener(QObject):
                     if time.perf_counter() - self.last_custom_mda > 0.2:
                         settings = evt.get_settings()
                         settings = MMSettings(java_settings=settings)
-                        print(settings)
                         self.mda_settings_event.emit(settings)
                     self.last_custom_mda = time.perf_counter()
                 elif "DefaultLiveModeEvent" in eventString:
@@ -204,21 +209,22 @@ class EventListener(QObject):
                         self.blockImages = evt.get_is_on()
                     self.live_mode_event.emit(self.blockImages)
                 elif "XYStagePositionChangedEvent" in eventString:
-                    self.xy_stage_position_changed_event.emit(
-                        (evt.get_x_pos(), evt.get_y_pos())
-                    )
+                    self.xy_stage_position_changed_event.emit((evt.get_x_pos(), evt.get_y_pos()))
             except zmq.error.Again:
                 self.timeouts += 1
                 # print("Server timeout", self.timeouts)
                 pass
 
-    def image_from_message(self, message, reply, image_depth):
-        image = np.frombuffer(message, dtype=image_depth)
+    def image_from_message(self, message, reply, image_depth, metadata_dict):
+        dt = np.dtype(image_depth)
+        dt = dt.newbyteorder('>')
+        image = np.frombuffer(message, dtype=dt)
         image_params = re.split("NewImage ", reply)[1]
         image_params = re.split(", ", image_params[1:-2])
         image_params = [int(round(float(x))) for x in image_params]
         py_image = PyImage(
             image.reshape([int(image_params[0]), int(image_params[1])]),
+            metadata_dict,
             *image_params[2:]
         )
         return py_image
